@@ -1,5 +1,5 @@
 /*  Clzip - LZMA lossless data compressor
-    Copyright (C) 2010-2017 Antonio Diaz Diaz.
+    Copyright (C) 2010-2018 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -29,24 +29,6 @@
 #include "decoder.h"
 
 
-void Pp_show_msg( struct Pretty_print * const pp, const char * const msg )
-  {
-  if( verbosity >= 0 )
-    {
-    if( pp->first_post )
-      {
-      unsigned i;
-      pp->first_post = false;
-      fprintf( stderr, "  %s: ", pp->name );
-      for( i = strlen( pp->name ); i < pp->longest_name; ++i )
-        fputc( ' ', stderr );
-      if( !msg ) fflush( stderr );
-      }
-    if( msg ) fprintf( stderr, "%s\n", msg );
-    }
-  }
-
-
 /* Returns the number of bytes really read.
    If (returned value < size) and (errno == 0), means EOF was reached.
 */
@@ -94,6 +76,7 @@ bool Rd_read_block( struct Range_decoder * const rdec )
     rdec->at_stream_end = ( rdec->stream_pos < rd_buffer_size );
     rdec->partial_member_pos += rdec->pos;
     rdec->pos = 0;
+    show_dprogress( 0, 0, 0, 0 );
     }
   return rdec->pos < rdec->stream_pos;
   }
@@ -122,6 +105,8 @@ static bool LZd_verify_trailer( struct LZ_decoder * const d,
   int size = Rd_read_data( d->rdec, trailer, Ft_size );
   const unsigned long long data_size = LZd_data_position( d );
   const unsigned long long member_size = Rd_member_position( d->rdec );
+  unsigned td_crc;
+  unsigned long long td_size, tm_size;
   bool error = false;
 
   if( size < Ft_size )
@@ -136,45 +121,56 @@ static bool LZd_verify_trailer( struct LZ_decoder * const d,
     while( size < Ft_size ) trailer[size++] = 0;
     }
 
-  if( Ft_get_data_crc( trailer ) != LZd_crc( d ) )
+  td_crc = Ft_get_data_crc( trailer );
+  if( td_crc != LZd_crc( d ) )
     {
     error = true;
     if( verbosity >= 0 )
       {
       Pp_show_msg( pp, 0 );
-      fprintf( stderr, "CRC mismatch; trailer says %08X, data CRC is %08X\n",
-               Ft_get_data_crc( trailer ), LZd_crc( d ) );
+      fprintf( stderr, "CRC mismatch; stored %08X, computed %08X\n",
+               td_crc, LZd_crc( d ) );
       }
     }
-  if( Ft_get_data_size( trailer ) != data_size )
+  td_size = Ft_get_data_size( trailer );
+  if( td_size != data_size )
     {
     error = true;
     if( verbosity >= 0 )
       {
       Pp_show_msg( pp, 0 );
-      fprintf( stderr, "Data size mismatch; trailer says %llu, data size is %llu (0x%llX)\n",
-               Ft_get_data_size( trailer ), data_size, data_size );
+      fprintf( stderr, "Data size mismatch; stored %llu (0x%llX), computed %llu (0x%llX)\n",
+               td_size, td_size, data_size, data_size );
       }
     }
-  if( Ft_get_member_size( trailer ) != member_size )
+  tm_size = Ft_get_member_size( trailer );
+  if( tm_size != member_size )
     {
     error = true;
     if( verbosity >= 0 )
       {
       Pp_show_msg( pp, 0 );
-      fprintf( stderr, "Member size mismatch; trailer says %llu, member size is %llu (0x%llX)\n",
-               Ft_get_member_size( trailer ), member_size, member_size );
+      fprintf( stderr, "Member size mismatch; stored %llu (0x%llX), computed %llu (0x%llX)\n",
+               tm_size, tm_size, member_size, member_size );
       }
     }
-  if( !error && verbosity >= 2 && data_size > 0 && member_size > 0 )
-    fprintf( stderr, "%6.3f:1, %6.3f bits/byte, %5.2f%% saved.  ",
-             (double)data_size / member_size,
-             ( 8.0 * member_size ) / data_size,
-             100.0 * ( 1.0 - ( (double)member_size / data_size ) ) );
-  if( !error && verbosity >= 4 )
-    fprintf( stderr, "CRC %08X, decompressed %9llu, compressed %8llu.  ",
-             LZd_crc( d ), data_size, member_size );
-  return !error;
+  if( error ) return false;
+  if( verbosity >= 2 )
+    {
+    if( verbosity >= 4 ) show_header( d->dictionary_size );
+    if( data_size == 0 || member_size == 0 )
+      fputs( "no data compressed.  ", stderr );
+    else
+      fprintf( stderr, "%6.3f:1, %5.2f%% ratio, %5.2f%% saved.  ",
+               (double)data_size / member_size,
+               ( 100.0 * member_size ) / data_size,
+               100.0 - ( ( 100.0 * member_size ) / data_size ) );
+    if( verbosity >= 4 ) fprintf( stderr, "CRC %08X, ", td_crc );
+    if( verbosity >= 3 )
+      fprintf( stderr, "decompressed %9llu, compressed %8llu.  ",
+               data_size, member_size );
+    }
+  return true;
   }
 
 
